library(mgcv)

load("dat3.rda")

# Sequential model comparison (cf. Table 3, p. 16):
dat3.gam0 = gam(PitchST ~ 1, data=dat3)
dat3.gam1 = gam(PitchST ~ 1 + s(speaker, bs="re"), data=dat3)
dat3.gam2 = gam(PitchST ~ 1 + s(speaker, bs="re") + s(item, bs="re"), data=dat3)
dat3.gam3 = gam(PitchST ~ 1 + s(speaker, bs="re") + s(item, bs="re") + Len, data=dat3)
dat3.gam4 = gam(PitchST ~ 1 + s(speaker, bs="re") + s(item, bs="re") + Len + position, data=dat3)
dat3.gam5 = gam(PitchST ~ 1 + s(speaker, bs="re") + s(item, bs="re") + Len * position, data=dat3)
dat3.gam6 = gam(PitchST ~ 1 + s(speaker, bs="re") + s(item, bs="re") + Len * position + branching, data=dat3)
dat3.gam7 = gam(PitchST ~ 1 + s(speaker, bs="re") + s(item, bs="re") + Len * position + branching + s(t, by=branching), data=dat3)
dat3.gam8 = gam(PitchST ~ 1 + s(speaker, bs="re") + s(item, bs="re") + Len * position + branching + s(t, by=branching) + sex, data=dat3)
dat3.gam9 = gam(PitchST ~ 1 + s(speaker, bs="re") + s(item, bs="re") + Len * position + branching + s(t, by=branching) + sex + s(t, by=sex), data=dat3)
dat3.gam10 = gam(PitchST ~ 1 + s(speaker, bs="re") + s(item, bs="re") + Len * position + branching + s(t, by=branching) + sex + s(t, by=sex) + Freq, data=dat3)
dat3.gam11 = gam(PitchST ~ 1 + s(speaker, bs="re") + s(item, bs="re") + Len * position + branching + s(t, by=branching) + sex + s(t, by=sex) + te(t, Freq), data=dat3)

tab = anova(dat3.gam0, dat3.gam1, dat3.gam2, dat3.gam3, dat3.gam4, dat3.gam5, dat3.gam6, dat3.gam7, dat3.gam8, dat3.gam9, dat3.gam10, dat3.gam11, test="F")

addedPreds = c("speaker", "item", "length", "position", "length:position", "branching", "s(t, branching)", "sex", "s(t, sex)", "Freq", "te(t, Freq)")

aicDiffs = -diff(c(AIC(dat3.gam0), AIC(dat3.gam1), AIC(dat3.gam2), AIC(dat3.gam3), AIC(dat3.gam4),
    AIC(dat3.gam5), AIC(dat3.gam6), AIC(dat3.gam7), AIC(dat3.gam8), AIC(dat3.gam9),
    AIC(dat3.gam10), AIC(dat3.gam11)))

tab = as.data.frame(tab) [-1,]
rownames(tab) = 1:nrow(tab)
tab$AICreduction = aicDiffs
tab = round(tab,4)
tab$AddedPredictor = addedPreds
tab = tab[,c("AddedPredictor", "Df", "AICreduction", "F", "Pr(>F)")]
colnames(tab) = c("Predictor", "Df", "Reduction AIC", "F", "p")

tab
#          Predictor      Df Reduction AIC          F      p
# 1          speaker 10.9998    92699.3300 34552.1873 0.0000
# 2             item 38.2940     1989.9877    70.0652 0.0000
# 3           length  1.0519       79.4889    98.4493 0.0000
# 4         position  1.0624     8741.8058  9533.9002 0.0000
# 5  length:position  1.0020       10.5634    13.2330 0.0003
# 6        branching  0.0325       -0.0324     1.0591 0.0555
# 7  s(t, branching) 34.5993     1871.0615    57.9695 0.0000
# 8              sex -0.0009        0.0004     1.5259 0.0029
# 9        s(t, sex)  6.7875      150.7458    24.4766 0.0000
# 10            Freq  1.0070       12.7048    14.7500 0.0001
# 11     te(t, Freq) 19.1058      502.5631    28.3969 0.0000

# final model (cf. Table 4, p. 17):
dat3.gamSept1 = bam(PitchST ~ 1 + s(speaker, bs="re") + s(item, bs="re") + Len * position + branching + s(t, by=branching) + sex + s(t, by=sex) + te(t, Freq), data=dat3)

summary(dat3.gamSept1)
#Parametric coefficients:
#              Estimate Std. Error t value Pr(>|t|)    
#(Intercept)  86.078720   3.270013  26.324  < 2e-16 ***
#Len           0.091407   0.019395   4.713 2.45e-06 ***
#position      0.215144   0.059531   3.614 0.000302 ***
#branchingLN2  1.715684   0.729530   2.352 0.018689 *  
#branchingRN2  1.316727   0.745807   1.766 0.077485 .  
#branchingRN3  3.109518   0.745132   4.173 3.01e-05 ***
#sexm         -9.863912   1.783592  -5.530 3.21e-08 ***
#Len:position -0.062879   0.008315  -7.562 4.03e-14 ***
#---
#Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1 
#

# Approximate significance of smooth terms:
#                         edf    Ref.df        F  p-value    
# s(speaker)        9.999e+00 10.000000 9238.366  < 2e-16 ***
# s(item)           3.536e+01 36.000000  108.594  < 2e-16 ***
# s(t):branchingLN1 8.288e+00  8.708646   18.466  < 2e-16 ***
# s(t):branchingLN2 7.626e+00  8.312292    8.371 1.15e-11 ***
# s(t):branchingRN2 8.320e+00  8.726427    7.830 3.31e-11 ***
# s(t):branchingRN3 8.367e+00  8.749189    7.409 1.58e-10 ***
# s(t):sexf         4.956e+00  6.011112   24.731  < 2e-16 ***
# s(t):sexm         8.785e-04  0.001123    0.024    0.996    
# te(t,Freq)        1.996e+01 20.799880   25.557  < 2e-16 ***
# ---
# Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1 
#
#R-sq.(adj) =  0.893   Deviance explained = 89.3%
#fREML score = 1.0045e+05  Scale est. = 3.9513    n = 47574

# helper function that determines average positions of constituent boundaries
# for a given branching:
my.getAvgPos <- function (branching, data=NULL) {
  avgPos <- c()
  # for compatibility:
  if (is.null (data)) 
    dat = dat2
  else
    dat = data
  sub <- dat[dat$branching == branching, ]
  sub <- sub [order(sub$speaker, sub$item, sub$t),]
  pos <- sub$position
  changePositions = which(pos[2:length(pos)] != pos[1:(length(pos)-1)]) 
  changeLabels = rep(c("12", "23", "junk"), floor(length(changePositions)/3))
  changeLabels = c(changeLabels, c("12", "23"))

  changes = data.frame(position = changePositions, type = changeLabels)
  changes$Time = sub$t[changePositions]
  changes = changes[changes$type != "junk",]
  changes$type = changes$type[drop=TRUE]

  return (tapply(changes$Time + 0.5, changes$type, mean))
}


# Plot estimated pitch contours by branching type (cf. Figure 4, p. 18):
dat3.gam = dat3.gamSept1
pdf("dat3Sept1.pdf", height=7, width=6)
    par(mfrow=c(2, 2), mar=c(4, 4, 3, 1))
    plot(dat3.gam, select=3, rug=FALSE, seWithMean=FALSE, shade=T, ylab="ST")
    mtext("LN1", 3, 0.5, cex=0.8)
    pos=my.getAvgPos("LN1", dat3)
    abline(v=pos, lty=2)
    plot(dat3.gam, select=4, rug=FALSE, seWithMean=FALSE, shade=T, ylab="ST")
    mtext("LN2", 3, 0.5, cex=0.8)
    pos=my.getAvgPos("LN2", dat3)
    abline(v=pos, lty=2)
    plot(dat3.gam, select=5, rug=FALSE, seWithMean=FALSE, shade=T, ylab="ST")
    mtext("RN2", 3, 0.5, cex=0.8)
    pos=my.getAvgPos("RN2", dat3)
    abline(v=pos, lty=2)
    plot(dat3.gam, select=6, rug=FALSE, seWithMean=FALSE, shade=T, ylab="ST")
    mtext("RN3", 3, 0.5, cex=0.8)
    pos=my.getAvgPos("RN3", dat3)
    abline(v=pos, lty=2)
    par(mfrow=c(1, 1))
dev.off()
